package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.XServiceResult;

/**
 * 光伏预测训练定时任务
 *
 * @author ZWT
 * @date 2023/10/19 13:59
 */
@XText("中期间开优化定时任务")
@XService
public class PhotovoltaicTrainJob {

    /**
     * 每三天执行    0 0 0 1/3 * ?
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ PhotovoltaicTrainJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "PhotovoltaicTrainJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                XThreadHelper.async(() -> {
                    try {
                        HttpUtil.get("http://127.0.0.1:10060/api/photovoltaic/train?secretKey=fZ1kC1aV8bO7rH3bL8oE5zI0mY3vW1sG");
                    } catch (Exception e) {
                        xContext.getLogger().error("---- train", e);
                    }
                });
            } catch (Exception e) {
                xContext.getLogger().error("------ PhotovoltaicTrainJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ PhotovoltaicTrainJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}