package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.prediction.service.IThirdDataAccessCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;

/**
 * 第三方光伏日发电量定时任务
 *
 * @author ZWT
 * @date 2024/03/19 10:07
 */
@XText("第三方光伏日发电量定时任务")
public class ThirdPhotovoltaicPowerJob {

    /**
     * 每天00:05  0 5 0 * * ?
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ ThirdPhotovoltaicPowerJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "ThirdPhotovoltaicPowerJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                IThirdDataAccessCloudService service = xContext.getBean(IThirdDataAccessCloudService.class);
                XServiceResult result = service.thirdPhotovoltaicPowerJob(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ ThirdPhotovoltaicPowerJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ ThirdPhotovoltaicPowerJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}