package pps.core.task.job;

import cn.hutool.core.util.ObjectUtil;
import pps.cloud.base.service.IBaseWeatherCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;

/**
 * 天气API数据邮件接收定时任务
 *
 * @author ZWT
 * @date 2024/04/22 17:24
 */
@XText("天气API数据邮件接收定时任务")
public class WeatherApiReceiveJob {

    /**
     * 早/晚九点十五分执行   0 15 9/12 * * ?
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ WeatherApiReceiveJob start:{}", System.currentTimeMillis());
        String key = TaskConstant.TASK_LOCK_KEY + "WeatherApiReceiveJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                IBaseWeatherCloudService service = xContext.getBean(IBaseWeatherCloudService.class);
                XServiceResult result = service.weatherApiDataProcess(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ WeatherApiReceiveJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ WeatherApiReceiveJob end:{}", System.currentTimeMillis());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}