package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;

/**
 * 日耗电日产液信息
 *
 * @author ZWT
 * @date 2024/03/18 14:52
 */
@XText("日耗电日产液信息定时任务")
public class WellTechDailyJob {

    /**
     * 每天0点执行   0 0 0 * * ?
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ WellTechDailyJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "WellTechDailyJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                IPlantPredictedPowerCloudService service = xContext.getBean(IPlantPredictedPowerCloudService.class);
                XServiceResult result = service.wellTechDailyJob(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ WellTechDailyJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ WellTechDailyJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}