package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.cloud.prediction.service.IStationDailyCloudService;
import pps.core.prediction.entity.StationDailyProductionSituationView;
import pps.core.prediction.entity.ThirdPhotovoltaicPowerEnt;
import pps.core.prediction.entity.WellheadDailyProductionSituationEnt;
import pps.core.prediction.mapper.StationDailyProductionSituationViewMapper;
import pps.core.prediction.mapper.ThirdPhotovoltaicPowerMapper;
import pps.core.prediction.mapper.WellheadDailyProductionSituationMapper;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 场站日统计
 *
 * @author ZWT
 * @date 2024/10/20 16:27
 */
@XService
public class StationDailyCloudServiceImpl implements IStationDailyCloudService {

    @Override
    public XServiceResult stationDailyJob(XContext context) {
        //查昨日井口统计数据
        DateTime yesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        WellheadDailyProductionSituationMapper wellDailyMapper = context.getBean(WellheadDailyProductionSituationMapper.class);
        List<WellheadDailyProductionSituationEnt> wellDailyList = wellDailyMapper.selectList(new LambdaQueryWrapper<WellheadDailyProductionSituationEnt>()
                .eq(WellheadDailyProductionSituationEnt::getDataDate, yesterday)
                .eq(WellheadDailyProductionSituationEnt::getSystemSource, oilFieldCode)
        );
        if (CollUtil.isEmpty(wellDailyList)) {
            return XServiceResult.OK;
        }
        //查线路关联井口
        List<DynamicQueryBasePowerLineWellheadViewOutput> lineWellheadList = ServiceUtil.getPowerLineWellheadList(context, new DynamicQueryBasePowerLineWellheadInput());
        if (CollUtil.isEmpty(lineWellheadList)) {
            return XServiceResult.OK;
        }
        //查日累积光伏发电量
        ThirdPhotovoltaicPowerMapper powerMapper = context.getBean(ThirdPhotovoltaicPowerMapper.class);
        List<ThirdPhotovoltaicPowerEnt> powerList = powerMapper.selectList(new LambdaQueryWrapper<ThirdPhotovoltaicPowerEnt>()
                .eq(ThirdPhotovoltaicPowerEnt::getCreateDate, yesterday)
                .eq(ThirdPhotovoltaicPowerEnt::getSystemSource, oilFieldCode)
        );
        Map<String, BigDecimal> powerMap;
        if (CollUtil.isEmpty(powerList)) {
            powerMap = Collections.emptyMap();
        } else {
            powerMap = powerList.stream()
                    .collect(Collectors.toMap(ThirdPhotovoltaicPowerEnt::getStationName, ThirdPhotovoltaicPowerEnt::getPhotovoltaicPower));
        }
        //按井号分组
        Map<String, WellheadDailyProductionSituationEnt> wellMap = wellDailyList.stream()
                .collect(Collectors.toMap(WellheadDailyProductionSituationEnt::getWellNumber, Function.identity()));
        //按线路分组
        Map<String, List<DynamicQueryBasePowerLineWellheadViewOutput>> lineMap = lineWellheadList.stream()
                .collect(Collectors.groupingBy(DynamicQueryBasePowerLineWellheadViewOutput::getLineName));
        //数据组装
        List<StationDailyProductionSituationView> dtoDailyList = new ArrayList<>(lineMap.size());
        StationDailyProductionSituationView view;
        DateTime now = DateUtil.date();
        List<DynamicQueryBasePowerLineWellheadViewOutput> wellList;
        WellheadDailyProductionSituationEnt wellDaily;
        for (Map.Entry<String, List<DynamicQueryBasePowerLineWellheadViewOutput>> entry : lineMap.entrySet()) {
            view = new StationDailyProductionSituationView();
            view.setStationName(entry.getKey());
            view.setDataDate(yesterday);
            view.setSystemSource(oilFieldCode);
            view.setCreateDate(now);
            view.setPhotovoltaicPower(powerMap.getOrDefault(entry.getKey(), BigDecimal.ZERO));
            view.setProductionTime(BigDecimal.ZERO);
            view.setDailyLiquidProduction(BigDecimal.ZERO);
            view.setDailyElectricityConsumption(BigDecimal.ZERO);
            view.setDailyUtilityElectricityConsumption(BigDecimal.ZERO);
            view.setElectricityConsumptionPh(BigDecimal.ZERO);
            view.setElectricityConsumptionHh(BigDecimal.ZERO);
            view.setElectricityConsumptionNp(BigDecimal.ZERO);
            view.setElectricityConsumptionVh(BigDecimal.ZERO);
            view.setElectricityChargeCity(BigDecimal.ZERO);
            view.setElectricityChargesPh(BigDecimal.ZERO);
            view.setPeakHourTariff(BigDecimal.ZERO);
            view.setNormalPeriodTariff(BigDecimal.ZERO);
            view.setAlleyHoursTariff(BigDecimal.ZERO);
            //累加数据
            wellList = entry.getValue();
            for (DynamicQueryBasePowerLineWellheadViewOutput well : wellList) {
                if (wellMap.containsKey(well.getWellNumber())) {
                    wellDaily = wellMap.get(well.getWellNumber());
                    view.setProductionTime(view.getProductionTime().add(wellDaily.getProductionTime()));
                    view.setDailyLiquidProduction(view.getDailyLiquidProduction().add(wellDaily.getDailyLiquidProduction()));
                    view.setDailyElectricityConsumption(view.getDailyElectricityConsumption().add(wellDaily.getDailyElectricityConsumption()));
                    view.setDailyUtilityElectricityConsumption(view.getDailyUtilityElectricityConsumption().add(wellDaily.getDailyUtilityElectricityConsumption()));
                    view.setElectricityConsumptionPh(view.getElectricityConsumptionPh().add(wellDaily.getElectricityConsumptionPh()));
                    view.setElectricityConsumptionHh(view.getElectricityConsumptionHh().add(wellDaily.getElectricityConsumptionHh()));
                    view.setElectricityConsumptionNp(view.getElectricityConsumptionNp().add(wellDaily.getElectricityConsumptionNp()));
                    view.setElectricityConsumptionVh(view.getElectricityConsumptionVh().add(wellDaily.getElectricityConsumptionVh()));
                    view.setElectricityChargeCity(view.getElectricityChargeCity().add(wellDaily.getElectricityChargeCity()));
                    view.setElectricityChargesPh(view.getElectricityChargesPh().add(wellDaily.getElectricityChargesPh()));
                    view.setPeakHourTariff(view.getPeakHourTariff().add(wellDaily.getPeakHourTariff()));
                    view.setNormalPeriodTariff(view.getNormalPeriodTariff().add(wellDaily.getNormalPeriodTariff()));
                    view.setAlleyHoursTariff(view.getAlleyHoursTariff().add(wellDaily.getAlleyHoursTariff()));

                }
            }
            dtoDailyList.add(view);
        }
        return XTransactionHelper.begin(context, () -> {
            StationDailyProductionSituationViewMapper dailyMapper = context.getBean(StationDailyProductionSituationViewMapper.class);
            //保存当日最新数据
            dailyMapper.batchInsert(dtoDailyList);
            return XServiceResult.OK;
        });
    }
}