package pps.core.middle.service;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import jakarta.validation.constraints.NotNull;
import pps.core.middle.entity.StationMappingEnt;
import pps.core.middle.mapper.StationMappingMapper;
import pps.core.middle.service.data.station_mapping.GetStationMappingInput;
import pps.core.middle.service.data.station_mapping.GetStationMappingOutput;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.io.File;
import java.io.FileWriter;
import java.math.BigDecimal;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@XService
public class StationMappingService {
    @XApiAnonymous
    @XApiGet(anonymous = true)
    public XSingleResult<GetStationMappingOutput> getStationMapping(XContext context, GetStationMappingInput input) {
        StationMappingMapper mapper = context.getBean(StationMappingMapper.class);
        List<StationMappingEnt> list = mapper.selectList(new LambdaQueryWrapper<>());
        for (StationMappingEnt stationMappingEnt : list) {
            context.getLogger().info(JSONUtil.toJsonStr(stationMappingEnt));
        }
        try {
            String storeDir = XStartup.getCurrent().getProperty("weather.file.temp.path");
            StringBuilder stringBuilder = new StringBuilder(storeDir)
                    .append(File.separator)
                    .append(DateTime.now().toString("yyyyMMdd"));
            String filePath = stringBuilder.append(File.separator).append("test.json").toString();
            String url = "http://192.168.100.128/tqfj/202410181200天气预测数据SY.json";
            HttpUtil.downloadFile(url, filePath);
            context.getLogger().info(filePath);
            context.getLogger().info(FileUtil.readUtf8String(filePath));
        } catch (Exception e) {
            context.getLogger().error(e.getMessage(), e);
        }
        return XSingleResult.success();
    }

    //Oracle数据库的ip和端口（可以是服务器上的）
    private static final String URL = "192.168.0.128:1521";
    //服务名称（SID）
    private static final String SERVICE_NAME = "ORCLPDB";
    //Oracle数据库用户名
    private static final String USERNAME = "DB_CONTRACT";
    //Oracle数据库密码
    private static final String PASSWORD = "DB_CONTRACT_123";
    //要导出脚本存储的位置
    private static final String PATH = "/home/gf-jar/temp/";
    private static final String SQL = "SELECT DBMS_METADATA.GET_DDL(U.OBJECT_TYPE, U.object_name) AS sql_statement, U.OBJECT_TYPE AS TABLE_FACTOR " +
            "    FROM USER_OBJECTS U " +
            "    WHERE U.OBJECT_TYPE = 'TABLE' " +
            "    OR U.OBJECT_TYPE = 'VIEW' " +
            "    OR U.OBJECT_TYPE = 'INDEX' " +
            "    OR U.OBJECT_TYPE = 'PROCEDURE' " +
            "    OR U.OBJECT_TYPE = 'SEQUENCE' " +
            "    OR U.OBJECT_TYPE = 'TRIGGER' " +
            "    ORDER BY U.OBJECT_TYPE DESC";


    @XApiAnonymous
    @XApiGet(anonymous = true)
    public XServiceResult test(XContext context) throws SQLException {
        List<String> list = new ArrayList<>();
        Connection conn = null;
        try {
            File saveFile = new File(PATH);
            if (!saveFile.exists()) {// 如果目录不存在
                saveFile.mkdirs();// 创建文件夹
            }
            Class.forName("oracle.jdbc.driver.OracleDriver");
            //三个参数的分别为连接的URL，用户名，密码
            //JDBC三件套
//            conn = DriverManager.getConnection("jdbc:oracle:thin:@" + URL + ":" + SERVICE_NAME, USERNAME, PASSWORD);
            conn = DriverManager.getConnection("jdbc:oracle:thin:@192.168.100.128:1521/ORCL", USERNAME, PASSWORD);
            PreparedStatement ps = conn.prepareStatement(SQL);
            ResultSet res = ps.executeQuery();
            String sql_statement = "";
            FileWriter writer;
            //在导出的脚本名称上加时间起到记录的作用,同时避免被覆盖,注意日期格式不要加空格（文件不支持）
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd_HH-mm-ss");
            String format = simpleDateFormat.format(new Date());
            writer = new FileWriter(PATH + "gqdc_" + format + ".sql");
            list.add(PATH);
            list.add("gqdc_" + format + ".sql");
            //得到的字符串较大,不能直接写入到脚本
            while (res.next()) {
                sql_statement = res.getString(1) + ";";
                writer.write(sql_statement);
            }
            writer.flush();
            writer.close();
            //查询某个用户（数据库）下所有表
            String sql = "select * from all_tables a where a.OWNER = upper('DB_CONTRACT')";
            ResultSet resultSet = ps.executeQuery(sql);
            while (resultSet.next()) {
                String Bname = getTableInsertSql(conn, resultSet.getString(2));
                list.add(Bname);
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            conn.close();
        }
        return XServiceResult.OK;
    }

    //导出单表,conn为连接，tableName为表名称
    public static String getTableInsertSql(@NotNull Connection conn, String tableName) throws Exception {
        ResultSet rs = null;
        Statement statement = null;
        //相当于拼接
        try {
            DatabaseMetaData metadata = conn.getMetaData();
            rs = metadata.getColumns(null, null, tableName, "%");    //得到表的字段列表
            String sql = "SELECT 'INSERT INTO " + tableName + " VALUES ( '";
            int count = 0;
            int counts = 0;
            //获得列的总数
            while (rs.next()) {
                count++;
            }
            //重新获得列数据 整理成sql
            rs = metadata.getColumns(null, null, tableName, "%");    //到表的字段列表
            while (rs.next()) {
                counts++;
                if (counts <= count) {
                    Object colName = rs.getObject("COLUMN_NAME");
                    BigDecimal dataType = (BigDecimal) rs.getObject("DATA_TYPE");
                    if (dataType.intValue() == 93 && ObjectUtil.isNotNull(colName)) {
                        sql += " || 'TO_DATE(''' || TO_CHAR( " + colName + ", 'yyyy-MM-dd HH24:mi:ss' ) || ''',''yyyy-MM-dd HH24:mi:ss'' ),'";
                    } else {
                        sql += " ||'''' ||" + colName + "|| ''','";
                    }
                }
            }
            sql = sql.substring(0, sql.length() - 2) + "'";
            sql += " || ' )' from " + tableName;
            rs.close();
            statement = conn.createStatement();
            rs = statement.executeQuery(sql);

            FileWriter writer;
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd_HH-ss-mm");
            String format = simpleDateFormat.format(new Date());
            writer = new FileWriter(PATH + tableName + "_" + format + ".sql");
            String sql_statement;
            while (rs.next()) {
                sql_statement = rs.getString(1);
                sql_statement += ";\n";
                writer.write(sql_statement);
            }
            writer.flush();
            writer.close();
            return tableName + "_" + format + ".sql";
        } finally {
            if (rs != null)
                rs.close();
            if (statement != null)
                statement.close();
        }
    }
}
